#!/usr/bin/env python

from vdb import *

# ------------------------------------------------------------------------

#exam the database what sub-databases, sub-tables
def exam_db( db ) :
    try :
        print( "database \t: %s"% ( db.Name() ) )
    except vdb_error as e :
        pass
    try :
        print( "sub-db-s \t: %s"%( db.ListDB() ) )
    except vdb_error as e :
        pass

# ------------------------------------------------------------------------

def exam_tables( db ) :
    try :
        sub_tabs = db.ListTbl()
        print( "sub-tables's \t: %s"%sub_tabs )
        for t in sub_tabs :
            tab = db.OpenTable( t )
            print( "\n%s.columns = %s"%( tab.Name(), tab.ListCol() ) )
    except vdb_error as e :
        pass

# ------------------------------------------------------------------------

def exam_index( db ) :
    try :
        for tablename in db.ListTbl() :
            t = db.OpenTable( tablename )
            index_list = t.ListIdx()
            print( "\nINDEX %s.%s : %s"%( db.Name(), tablename, index_list ) )
            if len( index_list ) > 0 :
                for index_name in index_list :
                    index = t.OpenIndexRead( index_name )
                    print( "Version( '%s' ) = %s"%( index_name, index.Version() ) )
                    print( "Type( '%s' ) = %s"%( index_name, index.Type() ) )
                    print( "Locked( '%s' ) = %s"%( index_name, index.Locked() ) )
    except vdb_error as e :
        pass

# ------------------------------------------------------------------------

if __name__ == '__main__' :
    try :
        #open database in read-mode ( default )
        with manager().OpenDB( "SRR834507" ) as db :
            exam_db( db )
            exam_tables( db )
            exam_index( db )

    except vdb_error as e :
        print( e )



import sys, os
from collections import namedtuple

KarFile = namedtuple( 'KarFile', 'spos epos name' )

def ingest_kar_list( kar_list ) :
    l = list()
    with open( kar_list ) as f :
        for line in f :
            v = line.strip().split()
            if v[ 0 ] == "-r--r--r--" and v[ 1 ] != '-' :
                size = int( v[ 1 ] )
                pos  = int( v[ 2 ] )
                name = v[ 5 ]
                kf = KarFile( pos, pos + size - 1, name )
                l.append( kf )
    return l

def lookup( l, pos ) :
    res = "?"
    for kf in l :
        if kf.spos <= pos and kf.epos >= pos :
            res = kf.name
            break
    return res
    
def annotate( rec_file, kar_list ) :
    l = ingest_kar_list( kar_list )
    with open( rec_file ) as f :
        for line in f :
            line_s = line.strip()
            v = line_s.split()
            if v[ 0 ] == 'R' :
                print( "%s\t%s"%( line_s, lookup( l, int( v[ 1 ] ) ) ) )
            else :
                print( line_s )

if __name__ == '__main__':
    if len( sys.argv ) > 2 :
        rec_file = sys.argv[ 1 ]
        kar_list = sys.argv[ 2 ]
        found = 0
        if not os.path.isfile( rec_file ) :
            print( "'%s' not found"%( rec_file ) )
        else :
            found += 1
        if not os.path.isfile( kar_list ) :
            print( "'%s' not found"%( kar_list ) )
        else :
            found += 1
        if found == 2 :
            annotate( rec_file, kar_list )

    else :
        print( "no input given" )
        print( "example: python annotate xxx.rec xxx.kar.list" )



#!/usr/bin/env python

from vdb import *

lib_wr = "./2.8.0/libncbi-wvdb.so.2.8.0"

schematxt = '''
version 1;

include 'vdb/vdb.vschema';

table A_TABLE #1.0
{
    column < U8 > izip_encoding C1;
};

''' 

def fill_table_random( cur, cols, row_count, value_count ) :
    for idx in xrange( 0, row_count ) :
        cur.OpenRow()
        cols[ "C1" ].write_rand( value_count, 255 )
        cur.CommitRow()
        cur.CloseRow()

def make_table( mgr, schema_txt, table_name ) :
    try :
        schema = mgr.MakeSchema( schema_txt )
        tbl = mgr.CreateTable( schema, "A_TABLE", table_name )
        cur = tbl.CreateCursor( OpenMode.Write )
        cols = cur.OpenColumns( [ "C1" ] )
        fill_table_random( cur, cols, 320, 1024 )
        cur.Commit()
    except vdb_error as e :
        print( e )

        
if __name__ == '__main__' :
    
    table_name = "L7"
        
    try :
        mgr = manager( OpenMode.Write, lib_wr )
    
        make_table( mgr, schematxt, table_name )
        #mgr.OpenTable( table_name ).print_rows()

    except vdb_error as e :
        print( e )



import unittest
from ngs import NGS
from ngs.ErrorMsg import ErrorMsg
from ngs.ReadCollection import ReadCollection
from ngs.ReferenceSequence import ReferenceSequence
from ngs.Alignment import Alignment
from ngs.Read import Read

PrimaryOnly           = "SRR1063272"
WithSecondary         = "SRR833251"
WithGroups            = "SRR822962"
WithCircularRef       = "SRR1769246"
SingleFragmentPerSpot = "SRR2096940";

def getRead(id):
    run = NGS.openReadCollection(PrimaryOnly)
    return run.getRead(id)

def getAlignment(id):
    run = NGS.openReadCollection(PrimaryOnly)
    return run.getAlignment(id)

def getSecondaryAlignment(id):
    run = NGS.openReadCollection(WithSecondary)
    return run.getAlignment(id)

def getReference():
    return NGS.openReadCollection(PrimaryOnly).getReference("supercont2.1")

def getReferenceSequence():
    return NGS.openReferenceSequence("NC_011752.1")

class Tests(unittest.TestCase):

    def fail(self):
        self.assertTrue(False)

    def test_open_success(self):
        run = NGS.openReadCollection(PrimaryOnly)

    def test_open_fail(self):
        try:
            run = NGS.openReadCollection("SRRsomejunk")
            self.fail()
        except ErrorMsg:
            pass

    def test_ReadCollection_getName(self):
        self.assertEqual(PrimaryOnly, NGS.openReadCollection(PrimaryOnly).getName())

    def test_ReadCollection_getReadGroup(self):
        gr = NGS.openReadCollection(PrimaryOnly).getReadGroup("C1ELY.6")

    def test_ReadCollection_getReadGroups(self):
        it = NGS.openReadCollection(PrimaryOnly).getReadGroups()

    def test_ReadCollection_getReferences(self):
        it = NGS.openReadCollection(PrimaryOnly).getReferences()

    def test_ReadCollection_getReference(self):
        ref = NGS.openReadCollection(PrimaryOnly).getReference("supercont2.1")

    def test_ReadCollection_hasReference(self):
        assert ( NGS.openReadCollection(PrimaryOnly).hasReference("supercont2.1") )
        assert ( not NGS.openReadCollection(PrimaryOnly).hasReference("non-existent acc") )

    def test_ReadCollection_getAlignment(self):
        al = NGS.openReadCollection(PrimaryOnly).getAlignment(PrimaryOnly + ".PA.1")

    def test_ReadCollection_getAlignments_Primary(self):
        alIt = NGS.openReadCollection(PrimaryOnly).getAlignments(Alignment.primaryAlignment)

    def test_ReadCollection_getAlignments_Secondary(self):
        alIt = NGS.openReadCollection(PrimaryOnly).getAlignments(Alignment.secondaryAlignment)

    def test_ReadCollection_getAlignments_all(self):
        alIt = NGS.openReadCollection(PrimaryOnly).getAlignments(Alignment.all)

    def test_ReadCollection_getAlignmentCount_PrimaryOnly(self):
        self.assertEqual(3987701, NGS.openReadCollection(PrimaryOnly).getAlignmentCount())

    def test_ReadCollection_getAlignmentCount_PrimaryOnly_Primary(self):
        self.assertEqual(3987701, NGS.openReadCollection(PrimaryOnly).getAlignmentCount(Alignment.primaryAlignment))

    def test_ReadCollection_getAlignmentCount_PrimaryOnly_Secondary(self):
        self.assertEqual(0, NGS.openReadCollection(PrimaryOnly).getAlignmentCount(Alignment.secondaryAlignment))

    def test_ReadCollection_getAlignmentCount_PrimaryOnly_All(self):
        self.assertEqual(3987701, NGS.openReadCollection(PrimaryOnly).getAlignmentCount(Alignment.all))

    def test_ReadCollection_getAlignmentCount_WithSecondary(self):
        self.assertEqual(178, NGS.openReadCollection(WithSecondary).getAlignmentCount())

    def test_ReadCollection_getAlignmentCount_WithSecondary_Primary(self):
        self.assertEqual(168, NGS.openReadCollection(WithSecondary).getAlignmentCount(Alignment.primaryAlignment))

    def test_ReadCollection_getAlignmentCount_WithSecondary_Secondary(self):
        self.assertEqual(10, NGS.openReadCollection(WithSecondary).getAlignmentCount(Alignment.secondaryAlignment))

    def test_ReadCollection_getAlignmentCount_WithSecondary_All(self):
        self.assertEqual(178, NGS.openReadCollection(WithSecondary).getAlignmentCount(Alignment.all))

    def test_ReadCollection_getAlignmentRange(self):
        # straddling primary and secondary alignments
        alIt = NGS.openReadCollection(WithSecondary).getAlignmentRange(166, 5)
        self.assertTrue(alIt.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.166", alIt.getAlignmentId())

    def test_ReadCollection_getRead(self):
        read = NGS.openReadCollection(PrimaryOnly).getRead(PrimaryOnly + ".R.1")
        self.assertEqual(PrimaryOnly + ".R.1", read.getReadId())

    def test_ReadCollection_getReads(self):
        readIt = NGS.openReadCollection(PrimaryOnly).getReads(Read.all)
        self.assertTrue(readIt.nextRead())
        self.assertEqual(PrimaryOnly + ".R.1", readIt.getReadId())

    def test_ReadCollection_getReadCount(self):
        self.assertEqual(2280633, NGS.openReadCollection(PrimaryOnly).getReadCount())

    def test_ReadCollection_getReadRange(self):
        readIt = NGS.openReadCollection(PrimaryOnly).getReadRange(2, 3)
        self.assertTrue(readIt.nextRead())
        self.assertEqual(PrimaryOnly + ".R.2", readIt.getReadId())


# Read

    def test_Read_getReadCategory_full(self):
        self.assertEqual(Read.fullyAligned, getRead(PrimaryOnly + ".R.1").getReadCategory())

    def test_Read_getReadCategory_partial(self):
        self.assertEqual(Read.partiallyAligned, getRead(PrimaryOnly + ".R.3").getReadCategory())

    def test_Read_getNumFragments(self):
        self.assertEqual(2, getRead(PrimaryOnly + ".R.1").getNumFragments())

    def test_Read_fragmentIsAligned_partial(self):
        read = NGS.openReadCollection(PrimaryOnly).getRead(PrimaryOnly + ".R.3")
        self.assertEqual(True, read.fragmentIsAligned(0))
        self.assertEqual(False, read.fragmentIsAligned(1))

# FragmentIterator
    def test_FragmentIterator_ThrowsBeforeNext(self):
        try:
            getRead(PrimaryOnly + ".R.1").getFragmentId()
            self.fail()
        except ErrorMsg:
            pass

    def test_FragmentIterator_Next(self):
        read = getRead(PrimaryOnly + ".R.1")
        self.assertTrue(read.nextFragment())
        read.getReadCategory() # does not throw

# Fragment
    def test_Read_getFragmentId(self):
        read = getRead(PrimaryOnly + ".R.1")
        self.assertTrue(read.nextFragment())
        self.assertEqual(PrimaryOnly + ".FR0.1", read.getFragmentId())

    def test_getFragmentBases(self):
        read = getRead(PrimaryOnly + ".R.1")
        self.assertTrue(read.nextFragment())
        self.assertTrue(read.nextFragment())
        self.assertEqual("GGTA", read.getFragmentBases(2, 4));

    def test_getFragmentQualities(self):
        read = getRead(PrimaryOnly + ".R.1")
        self.assertTrue(read.nextFragment())
        self.assertTrue(read.nextFragment())
        self.assertEqual("@DDA", read.getFragmentQualities(2, 4))


# Alignment

    def test_Alignment_getAlignmentId(self):
        self.assertEqual(PrimaryOnly + ".PA.1", getAlignment(PrimaryOnly + ".PA.1").getAlignmentId())

    def test_Alignment_getReferenceSpec(self):
        self.assertEqual("supercont2.1", getAlignment(PrimaryOnly + ".PA.1").getReferenceSpec())

    def test_Alignment_getMappingQuality(self):
        self.assertEqual(60, getAlignment(PrimaryOnly + ".PA.1").getMappingQuality())

    def test_Alignment_getReferenceBases(self):
        self.assertEqual("ACTCGACATTCTGTCTTCGACCTATCTTTCTCCTCTCCCAGTCATCGCCCAGTAGAATTACCAGGCAATGAACCACGGCCTTTCATCCCAACGGCACAGCA",
                      getAlignment(PrimaryOnly + ".PA.1").getReferenceBases())

    def test_Alignment_getReadGroup(self):
        self.assertEqual("C1ELY.6", getAlignment(PrimaryOnly + ".PA.1").getReadGroup())

    def test_Alignment_getReadId(self):
        self.assertEqual(PrimaryOnly + ".R.165753", getAlignment(PrimaryOnly + ".PA.5").getReadId())

    def test_Alignment_getFragmentId(self):
        self.assertEqual(PrimaryOnly + ".FA0.1", getAlignment(PrimaryOnly + ".PA.1").getFragmentId())

    def test_Alignment_getFragmentBases_Raw(self):
        self.assertEqual("TGGATGCTCTGGAAAATCTGAAAAGTGGTGTTTGTAAGGTTTGCTGGCTGCCCATATACCACATGGATGATGGGGCTTTCCATTTTAATGTTGAAGGAGGA",
                      getAlignment(PrimaryOnly + ".PA.4").getFragmentBases())

    def test_Alignment_getFragmentQualities_Raw(self):
        self.assertEqual("######AA>55;5(;63;;3@;A9??;6..73CDCIDA>DCB>@B=;@B?;;ADAB<DD?1*>@C9:EC?2++A3+F4EEB<E>EEIEDC2?C:;AB+==1",
                      getAlignment(PrimaryOnly + ".PA.4").getFragmentQualities())

    def test_Alignment_getFragmentBases_Clipped(self):
        self.assertEqual("CTTCAACATTAAAATGGAAAGCCCCATCATCCATGTGGTATATGGGCAGCCAGCAAACCTTACAAACACCACTTTTCAGATTTTCCAGAGCATCCA",
                      getAlignment(PrimaryOnly + ".PA.4").getClippedFragmentBases())

    def test_Alignment_getFragmentQualities_Clipped(self):
        self.assertEqual("#AA>55;5(;63;;3@;A9??;6..73CDCIDA>DCB>@B=;@B?;;ADAB<DD?1*>@C9:EC?2++A3+F4EEB<E>EEIEDC2?C:;AB+==1",
                      getAlignment(PrimaryOnly + ".PA.4").getClippedFragmentQualities())

    def test_Alignment_getAlignedFragmentBases(self):
        self.assertEqual("ATATGGGTTCACTCCAACAGTGAACCATTCCAAAAGACCTTGCCTGCGTGGCCATCTCCTCACAAACCCACCATCCCGCAACATCTCAGGTATCATACCTT",
                      getAlignment(PrimaryOnly + ".PA.2").getAlignedFragmentBases())

    def test_Alignment_getAlignmentCategory(self):
        self.assertEqual(Alignment.primaryAlignment, getAlignment(PrimaryOnly + ".PA.4").getAlignmentCategory())

    def test_Alignment_getAlignmentPosition(self):
        self.assertEqual(85, getAlignment(PrimaryOnly + ".PA.1").getAlignmentPosition())

    def test_Alignment_getAlignmentLength(self):
        self.assertEqual(101, getAlignment(PrimaryOnly + ".PA.1").getAlignmentLength())

    def test_Alignment_getIsReversedOrientation_False(self):
        self.assertFalse(getAlignment(PrimaryOnly + ".PA.1").getIsReversedOrientation())

    def test_Alignment_getIsReversedOrientation_True(self):
        self.assertTrue(getAlignment(PrimaryOnly + ".PA.2").getIsReversedOrientation())

    def test_Alignment_getSoftClip_None(self):
        al = getAlignment(PrimaryOnly + ".PA.1")
        self.assertEqual(0, al.getSoftClip(Alignment.clipLeft))
        self.assertEqual(0, al.getSoftClip(Alignment.clipRight))

    def test_Alignment_getSoftClip_Left(self):
        al = getAlignment(PrimaryOnly + ".PA.4")
        self.assertEqual(5, al.getSoftClip(Alignment.clipLeft))
        self.assertEqual(0, al.getSoftClip(Alignment.clipRight))

    def test_Alignment_getSoftClip_Right(self):
        al = getAlignment(PrimaryOnly + ".PA.10")
        self.assertEqual(0,  al.getSoftClip(Alignment.clipLeft))
        self.assertEqual(13, al.getSoftClip(Alignment.clipRight))

    def test_Alignment_getTemplateLength(self):
        self.assertEqual(201, getAlignment(PrimaryOnly + ".PA.1").getTemplateLength())

    def test_Alignment_getShortCigar_Unclipped(self):
        self.assertEqual("5S96M", getAlignment(PrimaryOnly + ".PA.4").getShortCigar(False))

    def test_Alignment_getShortCigar_Clipped(self):
        self.assertEqual("96M", getAlignment(PrimaryOnly + ".PA.4").getShortCigar(True))

    def test_Alignment_getLongCigar_Unclipped(self):
        self.assertEqual("5S1X8=1X39=1X46=", getAlignment(PrimaryOnly + ".PA.4").getLongCigar(False))

    def test_Alignment_getLongCigar_Clipped(self):
        self.assertEqual("1X8=1X39=1X46=", getAlignment(PrimaryOnly + ".PA.4").getLongCigar(True))

    def test_Alignment_hasMate_Primary_No(self):
        self.assertFalse(getAlignment(PrimaryOnly + ".PA.99").hasMate())

    def test_Alignment_hasMate_Primary_Yes(self):
        self.assertTrue(getAlignment(PrimaryOnly + ".PA.1").hasMate())

    def test_Alignment_hasMate_Secondary(self):
        self.assertFalse(getSecondaryAlignment(WithSecondary + ".SA.169").hasMate())

    def test_Alignment_getMateAlignmentId(self):
        self.assertEqual(PrimaryOnly + ".PA.2", getAlignment(PrimaryOnly + ".PA.1").getMateAlignmentId())

    def test_Alignment_getMateAlignmentId_Missing(self):
        try:
            getAlignment(PrimaryOnly + ".PA.99").getMateAlignmentId()
            self.fail()
        except ErrorMsg:
            pass

    def test_Alignment_getMateAlignmentId_SecondaryThrows(self):
        try:
            getSecondaryAlignment(WithSecondary + ".SA.172").getMateAlignmentId()
            self.fail()
        except ErrorMsg:
            pass

    def test_Alignment_getMateAlignment(self):
        self.assertEqual(PrimaryOnly + ".PA.2", getAlignment(PrimaryOnly + ".PA.1").getMateAlignment().getAlignmentId())

    def test_Alignment_getMateAlignment_Missing(self):
        try:
            getAlignment(PrimaryOnly + ".PA.99").getMateAlignment()
            self.fail()
        except ErrorMsg:
            pass

    def test_Alignment_getMateAlignment_SecondaryThrows(self):
        try:
            getSecondaryAlignment(WithSecondary +".SA.172").getMateAlignment ()
            self.fail()
        except ErrorMsg:
            pass

    def test_Alignment_getMateReferenceSpec(self):
        self.assertEqual("supercont2.1",  getAlignment(PrimaryOnly + ".PA.1").getMateReferenceSpec())

    def test_Alignment_getMateIsReversedOrientation_Yes(self):
        self.assertTrue(getAlignment(PrimaryOnly + ".PA.1").getMateIsReversedOrientation())

    def test_Alignment_getMateIsReversedOrientation_No(self):
        self.assertFalse(getAlignment(PrimaryOnly + ".PA.2").getMateIsReversedOrientation())

    def test_Alignment_isPaired_MultiFragmentsPerSpot(self):
        readCollection = NGS.openReadCollection(PrimaryOnly)
        alignment = readCollection.getAlignment(PrimaryOnly + ".PA.1")
        self.assertTrue(alignment.isPaired())

        alignment = readCollection.getAlignment(PrimaryOnly + ".PA.2")
        self.assertTrue(alignment.isPaired())

        # has unaligned mate
        alignment = readCollection.getAlignment (PrimaryOnly + ".PA.6")
        self.assertTrue(alignment.isPaired())

    def test_Alignment_isPaired_SingleFragmentPerSpot(self):
        readCollection = NGS.openReadCollection(SingleFragmentPerSpot)
        alignment = readCollection.getAlignment(SingleFragmentPerSpot + ".PA.1")
        self.assertFalse(alignment.isPaired())

# ReferenceSequence
    def test_ReferenceSequence_getCanonicalName(self):
        self.assertEqual("gi|218511148|ref|NC_011752.1|", getReferenceSequence().getCanonicalName())

    def test_ReferenceSequence_getIsCircular_Yes(self):
        self.assertTrue(getReferenceSequence().getIsCircular())

    def test_ReferenceSequence_getLength(self):
        self.assertEqual(72482, getReferenceSequence().getLength())

    def test_ReferenceSequence_getReferenceBases(self):
        self.assertEqual("ATAAA", getReferenceSequence().getReferenceBases(72482 - 5))

    def test_ReferenceSequence_getReferenceBases_Length(self):
        self.assertEqual("TACA", getReferenceSequence().getReferenceBases(4998, 4))

    def test_ReferenceSequence_getReferenceChunk(self):
        self.assertEqual("TAATA", getReferenceSequence().getReferenceChunk(5000 - 5, 5))

    def test_ReferenceSequence_getReferenceChunk_Length (self):
        self.assertEqual("TAATA", getReferenceSequence().getReferenceChunk(5000 - 5, 10))

# Reference
    def test_Reference_getCommonName(self):
        self.assertEqual("supercont2.1", getReference().getCommonName())

    def test_Reference_getCanonicalName(self):
        self.assertEqual("NC_000007.13", NGS.openReadCollection("SRR821492").getReference("chr7").getCanonicalName())

    def test_Reference_getIsCircular_No(self):
        self.assertFalse(getReference().getIsCircular())

    def test_Reference_getIsCircular_Yes(self):
        self.assertTrue(NGS.openReadCollection("SRR821492").getReference("chrM").getIsCircular())

    def test_Reference_getIsLocal_Yes(self):
        self.assertTrue(getReference().getIsLocal())

    def test_Reference_getIsLocal_No(self):
        self.assertFalse(NGS.openReadCollection("SRR821492").getReference("chrM").getIsLocal())

    def test_Reference_getLength(self):
        self.assertEqual(2291499, getReference().getLength())

    def test_Reference_getReferenceBases(self):
        self.assertEqual("ATCTG", getReference().getReferenceBases(2291499 - 5))

    def test_Reference_getReferenceBases_Length(self):
        self.assertEqual("GCGCTATGAC", getReference().getReferenceBases(9000, 10))

    def test_Reference_getReferenceChunk(self):
        self.assertEqual("CTAGG", getReference().getReferenceChunk(5000 - 5, 5))

    def test_Reference_getReferenceChunk_Length (self):
        self.assertEqual("GCGCTATGAC", getReference().getReferenceChunk(9000, 10))

    def test_Reference_getAlignment(self):
        self.assertEqual(PrimaryOnly + ".PA.1", getReference().getAlignment(PrimaryOnly + ".PA.1").getAlignmentId())

#TODO: getAlignmentCount
#TODO: getAlignmentCount_Filtered

#TODO: getPileups
#TODO: getPileupRange
#TODO: getPileupRange_Filtered

# ReferenceIterator

    def test_ReferenceIterator_ThrowBeforeNext(self):
        it = NGS.openReadCollection(PrimaryOnly).getReferences()
        try:
            it.getCommonName()
            self.fail()
        except ErrorMsg:
            pass

    def test_ReferenceIterator_Next(self):
        it = NGS.openReadCollection(PrimaryOnly).getReferences()
        self.assertTrue(it.nextReference())
        self.assertEqual("supercont2.1", it.getCommonName())

# AlignmentIterator from Reference (ReferenceWindow)
    def test_ReferenceWindow(self):
        it = NGS.openReadCollection(WithSecondary).getReference("gi|169794206|ref|NC_010410.1|").getAlignments(Alignment.all)
        self.assertTrue(it.nextAlignment())

        # the first 2 secondary alignments' locations on the list: #34, #61
        count = 1;
        while it.nextAlignment():
            if it.getAlignmentCategory() == Alignment.secondaryAlignment:
                break
            count += 1

        self.assertEqual(34, count)
        while it.nextAlignment():
            if it.getAlignmentCategory() == Alignment.secondaryAlignment:
                break
            count += 1

        self.assertEqual(61, count)

    def test_ReferenceWindow_Slice(self):
        it = NGS.openReadCollection(WithSecondary).getReference("gi|169794206|ref|NC_010410.1|").getAlignmentSlice(516000, 100000)
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.33", it.getAlignmentId())
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.34", it.getAlignmentId())
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".SA.169", it.getAlignmentId()) #secondary
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.35", it.getAlignmentId())
        self.assertFalse(it.nextAlignment())

    def test_ReferenceWindow_Slice_Filtered_Category (self):
        it = NGS.openReadCollection(WithSecondary).getReference("gi|169794206|ref|NC_010410.1|").getAlignmentSlice(516000, 100000, Alignment.primaryAlignment)
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.33", it. getAlignmentId())
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.34", it. getAlignmentId())
        self.assertTrue(it.nextAlignment())
        self.assertEqual(WithSecondary + ".PA.35", it. getAlignmentId()) # no secondary
        self.assertFalse(it.nextAlignment())

    def test_ReferenceWindow_Slice_Filtered_Start_Within_Slice (self):
        ref = NGS.openReadCollection(WithCircularRef).getReference("NC_012920.1")
        it = ref.getFilteredAlignmentSlice(0, ref.getLength(), Alignment.all, Alignment.startWithinSlice, 0)

        self.assertTrue(it.nextAlignment())
        lastAlignmentPosition = it.getAlignmentPosition()
        while it.nextAlignment():
            currentPosition = it.getAlignmentPosition()
            errorMsg = "Sorting violated. Last position (" + str(lastAlignmentPosition) + ") is higher than current one (" + str(currentPosition) + ")"
            self.assertTrue ( lastAlignmentPosition <= currentPosition, errorMsg )
            lastAlignmentPosition = currentPosition

    # ReadGroup
    def test_ReadGroup_getName(self):
        gr = NGS.openReadCollection(PrimaryOnly).getReadGroup("C1ELY.6")
        self.assertEqual("C1ELY.6", gr.getName())

    def test_ReadGroup_has(self):
        assert ( NGS.openReadCollection(PrimaryOnly).hasReadGroup("C1ELY.6") )
        assert ( not NGS.openReadCollection(PrimaryOnly).hasReadGroup("non-existent read group") )

    def test_ReadGroup_getStatistics(self):
        gr = NGS.openReadCollection(WithGroups).getReadGroup("GS57510-FS3-L03")

        stats = gr.getStatistics()

        self.assertEqual(34164461870, stats.getAsU64("BASE_COUNT"))
        self.assertEqual(34164461870, stats.getAsU64("BIO_BASE_COUNT"))
        self.assertEqual(488063741,   stats.getAsU64("SPOT_COUNT"))
        self.assertEqual(5368875807,  stats.getAsU64("SPOT_MAX"))
        self.assertEqual(4880812067,  stats.getAsU64("SPOT_MIN"))

    # def test_ReadGroup_getRead(self):
        # gr = NGS.openReadCollection(PrimaryOnly).getReadGroup("C1ELY.6")
        # r = gr.getRead(PrimaryOnly + ".R.1")
        # self.assertEqual("C1ELY.6", r.getReadGroup())

    # def test_ReadGroup_getReads(self):
        # gr = NGS.openReadCollection(PrimaryOnly).getReadGroup("C1ELY.6")
        # r = gr.getReads(Read.partiallyAligned)

    # ReadGroupIterator
    def test_ReadGroupIterator_ThrowBeforeNext(self):
        it = NGS.openReadCollection(PrimaryOnly).getReadGroups()
        try:
            it.getName()
            self.fail()
        except ErrorMsg:
            pass

    def test_ReadGroupIterator_Next(self):
        it = NGS.openReadCollection(PrimaryOnly).getReadGroups();
        self.assertTrue(it.nextReadGroup());
        name = it.getName();

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

from vdb import *


if __name__ == '__main__':
    lib_262_r = "./2.8.0/libncbi-vdb.so.2.8.0"
    tablename = "SRR000001"
    
    try :
        mgr = manager( OpenMode.Read, lib_262_r )
        tab = mgr.open_tab( tablename )
        cur = tab.make_cursor( OpenMode.Read )
        cols = cur.open( [ "NAME", "READ", "(INSDC:quality:text:phred_33)QUALITY" ] )
        c_name = cols[ "NAME" ]
        c_read = cols[ "READ" ]
        c_qual = cols[ "(INSDC:quality:text:phred_33)QUALITY" ]
        for row in xrange( 1, 11 ) :
            name  = c_name.read( row )
            read  = c_read.read( row )
            print "@%s.%d %s lenght=%d"%( tablename, row, name, len( read ) )
            print read
            print "+%s.%d %s lenght=%d"%( tablename, row, name, len( read ) )
            print c_qual.read( row )
            
    except vdb_error, e :
        print e



import sys, os
from collections import namedtuple

Point = namedtuple( 'Point', 'x y' )

rect_fmt = "<rect x=\"%d\" y=\"%f\" width=\"%d\" height=\"%f\" style=\"fill:%s; stroke:none\" />\n"
line_fmt = "<line x1=\"%d\" y1=\"%f\" x2=\"%d\" y2=\"%f\" style=\"stroke:black; stroke-width:0.3\" />\n"
svg_fmt  = "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:ev=\"http://www.w3.org/2001/xml-events\" width=\"%d\" height=\"%d\">\n"

def svg_rect( f, p, w, h, color ) :
    f.write( rect_fmt%( p.x, p.y, w, h, color ) )

def svg_line( f, p1, p2 ) :
    f.write( line_fmt%( p1.x, p1.y, p2.x, p2.y ) )

def inspect( filename ) :
    if os.path.isfile( filename ) :
        print( "visualizing: %s"%( filename ) )

        max_pos = 0
        a_pos = []    # column NR 1  : request-position
        a_len = []    # column NR 2  : request-length

        with open( filename ) as f :
            for line in f :
                v = line.strip().split()
                if v[ 0 ] == "R" and len( v ) > 3:
                    pos = int( v[ 1 ] )
                    if pos > max_pos :
                        max_pos = pos;
                    a_pos.append( pos )
                    a_len.append( int( v[ 2 ] ) )
            f.close()

        with open( "%s.svg"%( filename ), "w" ) as f :

            ofs = 10
            divisor = 100000.0
    
            # calculate the extend of the svg-graphic
            svg_w = ( len( a_pos ) * 3 ) + ( 2 * ofs )
            svg_h = ( max_pos / divisor ) + ( 2 * ofs )
            f.write( svg_fmt%( svg_w, svg_h ) );

            p = Point( ofs, ofs )
            for i in range( len( a_pos ) ) :
            
                p1 = Point( p.x, a_pos[ i ] / divisor + ofs )
                h = a_len[ i ] / divisor
                
                svg_line( f, p, p1 )
                svg_rect( f, p1, 3, h, 'red' )

                p = Point( p1.x + 3, p1.y )

            f.write( "</svg>\n" )
            f.close()

if __name__ == '__main__':
    if len( sys.argv ) > 1 :
        inspect( sys.argv[ 1 ] )
    else :
        print( "no input given" )
        print( "example: python visual_request.py SRR341578.rec" )



#!/usr/bin/env python

from vdb import *

lib_262_rd = "./2.6.2/libncbi-vdb.so.2.6.2"
lib_262_wr = "./2.6.2/libncbi-wvdb.so.2.6.2"

lib_263_rd = "./2.6.3/libncbi-vdb.so.2.6.3"
lib_263_wr = "./2.6.3/libncbi-wvdb.so.2.6.3"

lib_270_rd = "./2.7.0/libncbi-vdb.so.2.7.0"
lib_270_wr = "./2.7.0/libncbi-wvdb.so.2.7.0"

lib_280_rd = "./2.8.0/libncbi-vdb.so.2.8.0"
lib_280_wr = "./2.8.0/libncbi-wvdb.so.2.8.0"

acc = "SRR000001"

if __name__ == '__main__' :

    rd_libs = [ lib_262_rd, lib_263_rd, lib_270_rd, lib_280_rd ]
    wr_libs = [ lib_262_wr, lib_263_wr, lib_270_wr, lib_280_wr ]
    
    for lib in rd_libs :
        try :
            mgr = manager( OpenMode.Read, lib )
            print( "%s\tmgr.Version() = %s\tmgr.writable() = %r"%( lib, mgr.Version(), mgr.writable() ) )
        except vdb_error as e :
            print( e )

    for lib in wr_libs :
        try :
            mgr = manager( OpenMode.Write, lib )
            print( "%s\tmgr.Version() = %s\tmgr.writable() = %r"%( lib, mgr.Version(), mgr.writable() ) )
        except vdb_error as e :
            print( e )

    try :
        # if the lib is omitted, the manager is looking for a lib itself
        # based on platform/OpenMode in the current directory first, then in $(HOME)/.ncbi/lib64
        # if no lib can be found or the lib cannot be loaded, then the constructor throws an exception
        mgr = manager()

        mv = mgr.Version()
        pt = mgr.PathType( acc )
        ov = mgr.GetObjVersion( acc )
        ot = mgr.GetObjModDate( acc )
        print( "mgr.vers\t%s\t%s\t%s\t Version %s\tObjModDate %s"%( mv, acc, pt, ov, ot ) )

        repo_mgr = mgr.MakeKConfig().MakeRepositoryMgr()
        print( "repo_mgr.HasRemoteAccess ... %r"%( repo_mgr.HasRemoteAccess() ) )
        for repo_list in repo_mgr.AllRepos() :
            for repo in repo_list :
                print( repo )
        
    except vdb_error as e :
        print( e )



#!/usr/bin/env python

import sys
import os.path
from vdb import *

schematxt = '''
version 1;

include 'insdc/insdc.vschema';

extern function
INSDC:quality:phred NCBI:SRA:syn_quality #1
    < INSDC:quality:phred good_quality, INSDC:quality:phred bad_quality >
    ( INSDC:coord:len read_len, INSDC:SRA:spot_filter spot_filter );

table TEST_TBL #1.0
{
    column INSDC:coord:len READ_LEN;
    column INSDC:SRA:spot_filter SPOT_FILTER;
    
    INSDC:coord:len out_read_len = .READ_LEN;
    INSDC:SRA:spot_filter out_spot_filter = .SPOT_FILTER;
    
    readonly column INSDC:quality:phred QUALITY =
        NCBI:SRA:syn_quality #1 < 30, 3 > ( out_read_len, out_spot_filter );
};

''' 

def make_rows( cur, cols, row_count, read_len_data, filter_data ) :
    for idx in xrange( 0, row_count ) :
        cur.OpenRow()
        cols[ "READ_LEN" ].write( read_len_data )
        cols[ "SPOT_FILTER" ].write( filter_data )
        cur.CommitRow()
        cur.CloseRow()
    cur.Commit()

def make_table( mgr, schema_txt, schema_table_name, table_name ) :
    try :
        schema = mgr.MakeSchema( schema_txt )
        tbl = mgr.CreateTable( schema, schema_table_name, table_name )
        cur = tbl.CreateCursor( OpenMode.Write )
        cols = cur.OpenColumns( [ "READ_LEN", "SPOT_FILTER" ] )
        row_count = 1
        make_rows( cur, cols, row_count, [ 9, 8, 7 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 8, 7, 6 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 7, 6, 5 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 6, 5, 4 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 5, 4, 3 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 4, 3, 2 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 3, 2, 1 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 1, 2, 3 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 2, 3, 4 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 3, 4, 5 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 4, 5, 6 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 6, 7, 8 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 7, 8, 9 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 8, 9, 9 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 9, 9, 9 ], [] )

        make_rows( cur, cols, row_count, [ 9, 8 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 8, 7 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 7, 6 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 6, 5 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 5, 4 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 4, 3 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 3, 2 ], [] )

        make_rows( cur, cols, row_count, [ 9 ], [ 0 ] )
        make_rows( cur, cols, row_count, [ 8 ], [ 1 ] )
        make_rows( cur, cols, row_count, [ 7 ], [] )

        make_rows( cur, cols, row_count, [], [ 0 ] )
        make_rows( cur, cols, row_count, [], [ 1 ] )
        make_rows( cur, cols, row_count, [], [] )
        
    except vdb_error as e :
        print( e )

        
if __name__ == '__main__' :

    argc = len( sys.argv )
    
    schema_table_name = "TEST_TBL"
    table_name = "T_TEST_SYN_QUAL"
    lib_name = None
    
    if argc > 1 : lib_wr = sys.argv[ 1 ]
    if argc > 2 : table_name = sys.argv[ 2 ]

    if ( lib_wr != None ) and ( os.path.isfile( lib_wr ) ) :
        try :
            mgr = manager( OpenMode.Write, lib_wr )
            make_table( mgr, schematxt, schema_table_name, table_name )
            #mgr.OpenTable( table_name ).print_rows()

        except vdb_error as e :
            print( e )



#!/usr/bin/env python

from vdb import *

lib_rd = './2.8.0/libncbi-vdb.so.2.8.0'

# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
        
if __name__ == '__main__' :
    
    try :
        #open a manager in read-mode
        mgr = manager( OpenMode.Read, lib_rd )
        
        #open accession ( a table-accession )
        tbl = mgr.OpenTable( 'SRR942391' )
       
        #open meta-data on table
        meta = tbl.OpenMetadata()
        
        print( "version = %s"%meta.Version() )
        print( "byte-order reversed = %r"%meta.ByteOrder() )
        print( "revision = %d"%meta.Revision() )
        print( "max-revision = %d"%meta.MaxRevision() )
        
        #open the root
        root_node = meta.OpenNode( '/' )
        child_list = root_node.ListChildren()
        print( "sub-nodes: %s"%child_list )
        for node_name in child_list :
            node = root_node.OpenNode( node_name )
            print ( "%s.byte-order reversed = %s"%( node_name, node.ByteOrder() ) )
            print ( "%s.size = %d"%( node_name, node.size() ) )
            print ( "%s.data = %s"%( node_name, node.as_string( 41 ) ) )
            print ( "%s.data (uint8)  = %s"%( node_name, node.as_uint8( 41 ) ) )
            print ( "%s.data (int8)   = %s"%( node_name, node.as_int8( 41 ) ) )
            print ( "%s.data (uint16) = %s"%( node_name, node.as_uint16( 41 ) ) )
            print ( "%s.data (int16)  = %s"%( node_name, node.as_int16( 41 ) ) )
            print ( "%s.data (uint32) = %s"%( node_name, node.as_uint32( 41 ) ) )
            print ( "%s.data (int32)  = %s"%( node_name, node.as_int32( 41 ) ) )
            print ( "%s.data (uint64) = %s"%( node_name, node.as_uint64( 41 ) ) )
            print ( "%s.data (int64)  = %s"%( node_name, node.as_int64( 41 ) ) )

    except vdb_error as e :
        print( e )



#!/usr/bin/env python

from vdb import *

lib_wr = "./2.8.0/libncbi-wvdb.so.2.8.0"

schema_txt = '''
version 1;

table SUB_TAB #1.0
{
    column U32 C1;
    column ascii C2;
};

database SUB_DB #1.0
{
    table SUB_TAB #1 TAB20;
};

database MAIN_DB #1.0
{
    table SUB_TAB #1 TAB10;
    database SUB_DB #1 SUBDB;
};
''' 

def fill_table( tbl, row_count, value_count ) :
    try :
        cur = tbl.CreateCursor( OpenMode.Write )
        cols = cur.OpenColumns( [ "C1", "C2" ] )
        for idx in xrange( 0, row_count ) :
            cur.OpenRow()
            cols[ "C1" ].write_rand( value_count, 100 )
            cols[ "C2" ].write_rand( value_count, 80 )
            cur.CommitRow()
            cur.CloseRow()
        cur.Commit()
    except vdb_error as e :
        print( e )

def create_database( mgr, spec, path ) :
    #make a schema from the text above
    schema = mgr.MakeSchema( schema_txt )

    #create a database with this schema
    spec_in_schema = "MAIN_DB"
    path_to_create = "L5"
    db = mgr.CreateDB( schema, spec, path )
    
    t1 = db.CreateTable( "TAB10" )
    fill_table( t1, 2, 3 )
    
    subdb = db.CreateDB( "SUBDB" )
    t2 = subdb.CreateTable( "TAB20" )
    fill_table( t2, 2, 3 )

# ------------------------------------------------------------------------

def print_tables( db, prefix ) :
    try :
        tables = db.ListTbl()
        for tabname in tables :
            try :
                t = db.OpenTable( tabname )
                print( "%s|---TABLE: '%s'"%( prefix, t.Name() ) )
                t.print_rows( None, None, "%s    "%( prefix ) )
            except vdb_error as e :
                print( e )
    except vdb_error as e :
        pass

def print_databases( db, prefix ) :
    print_tables( db, prefix )
    try :
        databases = db.ListDB()
        for dbname in databases :
            try :
                d = db.OpenDB( dbname )
                print( "%s|---DATABASE: '%s'"%( prefix, d.Name() ) )
                print_databases( d, "%s    "%( prefix ) )
            except vdb_error as e :
                print( e )
    except vdb_error as e :
        pass

def print_db( db ) :
    print( "DATABASE: '%s':"%( db.Name() ) )
    print_databases( db, "" )

# ------------------------------------------------------------------------
        
if __name__ == '__main__' :
    
    try :
        #open a manager in read-mode
        mgr = manager( OpenMode.Write, lib_wr )
        
        #make a schema from the text above
        schema = mgr.MakeSchema( schema_txt )

        #create a database with this schema
        spec_in_schema = "MAIN_DB"
        path_to_create = "L5"
        create_database( mgr, spec_in_schema, path_to_create )
        
        #print the content of the created database
        print_db( mgr.OpenDB( path_to_create ) )
       
    except vdb_error as e :
        print( e )



#!/usr/bin/env python

from vdb import *
import multiprocessing, sys

'''
def f1_a( mgr, ref, del_pos, del_len, ins ) :
    del_pos = 2
    del_len = 1
    ins = "CCAA"
    
    print "ref    = '%s' del: %d:%d ins: '%s'" % ( ref, del_pos, del_len, ins )
    
    ref_var = mgr.make_ref_var( ref, del_pos, del_len, ins )
    
    #q  = ref_var.search()
    #qr = ref_var.search_len()
    #print "query  = '%s' at: %d:%d, on ref: %d" %( q[0], q[2], q[1], qr )
    
    ( a_bases, a_len, a_pos ) = ref_var.allele()
    ar = ref_var.allele_len()
    print "allele = '%s' at: %d:%d, on ref: %d" %( a_bases, a_pos, a_len, ar )

#------------------------------------------------------------------------------------------------------------
def f1( mgr ):
    ref = "ACCGGTTAACC"
    
    del_pos = 2
    del_len = 1
    ins = "CCAA"

    f1_a( mgr, ref, del_pos, del_len, ins )
    f1_a( mgr, ref, del_pos, del_len, ins )    

#------------------------------------------------------------------------------------------------------------
def f2( mgr, acc ) :
    refs = mgr.make_reflist( acc )
    count = refs.count()
    print "we have %d references:" % count
    for idx in xrange( count ) :
        print "\t No. %d" % idx
        obj = refs.get( idx )
        #print "\t\tidx:\t%d" % obj.get_idx()
        rr = obj.get_row_range()
        print "\t\trows:\t%d..%d" % ( rr[0], rr[1] )
        #print "\t\tbin:\t%d" % obj.get_bin()
        print "\t\tSeqId:\t%s" % obj.get_seq_id()
        print "\t\tname:\t%s" % obj.get_name()
        print "\t\tlength:\t%d" % obj.get_length()
        print "\t\tcirc:\t%s" % obj.is_circular()
        print "\t\text:\t%s" % obj.is_external()
        #print "\t\tdata:\t%s" % obj.read( 0, 50 )
        #print "\t\tid-cnt:\t%d" % obj.id_count( rr[ 0 ] )


#------------------------------------------------------------------------------------------------------------
def num( s ):
    try:
        return int( s )
    except ValueError:
        return 0


#------------------------------------------------------------------------------------------------------------
def split_cigar( cigar ) :
    res = list()
    op_len = ""
    for i in xrange( 0, len( cigar ) ) :
        op = cigar[ i ]
        if op >= '0' and op <= '9' :
            op_len = op_len + op
        else :
            tup = ( num( op_len ), op )
            op_len = ""
            res.append( tup )
    return res


#------------------------------------------------------------------------------------------------------------
def f3( mgr, acc ) :
    cur = mgr.open_db( acc ).open_tab( "PRIMARY_ALIGNMENT" ).make_cursor()
    cols = cur.open( [ "CIGAR_SHORT", "READ", "REF_SEQ_ID", "REF_POS", "REF_LEN" ] )
    row = read_row( cols, 1 )
    print row


#------------------------------------------------------------------------------------------------------------
def f4( mgr, acc ) :
    cur = mgr.open_db( acc ).open_tab( "PRIMARY_ALIGNMENT" ).make_cursor()
    cols = cur.open( [ "CIGAR_SHORT", "READ", "REF_SEQ_ID", "REF_POS", "REF_LEN" ] )
    row_range = cols[ "READ" ].range()
    print row_range
    for row in row_range :
        row_data = read_row( cols, row )
        if row % 1000 == 0 :
            sys.stdout.write( '.' )
            sys.stdout.flush()

#------------------------------------------------------------------------------------------------------------
def cigar_splitter( cigar, ref_pos, reference ) :
    ops = split_cigar( cigar )
    

#------------------------------------------------------------------------------------------------------------
def handle_reference( ref_obj, total, prim_alig_id_col, prim_cols, cigars ) :
    res = total
    ref_rows = ref_obj.get_row_range()
    ref_len = ref_obj.get_length()
    name = ref_obj.get_seq_id()
    #read the whole reference in...
    reference = ref_obj.read( 0, ref_len )
    print "\n", name, ref_len, len( reference )
    # for each row in the reference-table of this reference
    for ref_row in xrange( ref_rows[ 0 ], ref_rows[ 1 ] + 1 ) :
        prim_ids = prim_alig_id_col.read( ref_row )
        #for each alignment in this reference-block
        for prim_id in prim_ids :
            row_data = read_row( prim_cols, prim_id )
            cigar = row_data[ "CIGAR_SHORT" ]
            if cigar in cigars.keys() :
                cigars[ cigar ] += 1
            else :
                cigars[ cigar ] = 1
            res += 1
            if res % 1000 == 0 :
                sys.stdout.write( '.' )
                sys.stdout.flush()
    return res


  
#------------------------------------------------------------------------------------------------------------
def f5( mgr, acc, ref_idx = None ) :
    db = mgr.open_db( acc )
    cur_a = db.open_tab( "PRIMARY_ALIGNMENT" ).make_cursor()
    cur_r = db.open_tab( "REFERENCE" ).make_cursor()
    refs = db.make_reflist()
    prim_cols = cur_a.open( [ "CIGAR_SHORT", "READ", "REF_SEQ_ID", "REF_POS", "REF_LEN" ] )
    prim_alig_id_col = cur_r.open( "PRIMARY_ALIGNMENT_IDS" )
    total = 0
    cigars = {}
    
    if ref_idx == None :
    # for each reference
        for idx in xrange( refs.count() ) :
            total += handle_reference( refs.get( idx ), total, prim_alig_id_col, prim_cols, cigars )       
    else :
        total += handle_reference( refs.get( ref_idx ), total, prim_alig_id_col, prim_cols, cigars )       
    
    print "\nhandled ", total, " alignments"
    print "we have ", len( cigars ), " different cigar-strings"
    sorted_cigars = sorted( cigars, key = cigars.get, reverse = True )
    for w in sorted_cigars[ 0 : 10 ] :
        print w, cigars[ w ]

'''

#------------------------------------------------------------------------------------------------------------
def cigar2events( cigar ) :
    res = list()
    tmp = ""
    for c in cigar :
        if c >= '0' and c <= '9' :
            tmp += c
        else :
            res.append( ( int( tmp ), c ) )
            tmp = ""
    return res


#------------------------------------------------------------------------------------------------------------
def events2dict( events ) :
    res = {}
    for ( len, op ) in events :
        res[ op ] = res.get( op, 0 ) + 1
    return res


#------------------------------------------------------------------------------------------------------------
def adjacent( events, op1, op2 ) :
    res = 0
    last = 'x'
    for ( len, op ) in events :
        if op == op1 and last == op2 :
            res += 1
        if op == op2 and last == op1 :
            res += 1
        last = op
    return res


#------------------------------------------------------------------------------------------------------------
def cigar_events( cigar, read, refname, pos ) :
    ref_pos  = pos
    ali_pos = 0
    events = cigar2events( cigar )
    for ( len, op ) in events :
        if op == '=' :          # we have a perfect match between reference and alignment '='
            ref_pos += len      # we advance on alignment AND reference
            ali_pos += len

        elif op == 'X' :        #we have a mismatch  between reference and alignment 'X'
            yield ( refname, ref_pos, 0, read[ ali_pos : ali_pos + len ] )
            ref_pos += len      # we advance on alignment AND reference
            ali_pos += len

        elif op == 'D' :        #we have a deletion on the reference 'D'
            yield ( refname, ref_pos, len, '' )
            ref_pos += len      # we advance only on reference

        elif op == 'I' :        #we have a insertion on the reference 'I'
            yield ( refname, ref_pos, 0, read[ ali_pos : ali_pos + len ] )
            ali_pos += len      # we advance only on alignment
            
        elif op == 'S' :        #we have a soft clip 'S'
            ali_pos += len      # we advance only on alignment

        elif op == 'H' :        #we have a hard clip 'H'
            ali_pos += 0        # we do nothing

        elif op == 'N' :        #we have a reference 'N'
            ref_pos += len      # we advance on alignment AND reference

        else :
            ali_pos += 0        # we do nothing

#------------------------------------------------------------------------------------------------------------
# a generator of alleles...
def alleles( db, row_range = None ) :
    column_list = [ 'CIGAR_LONG', 'READ', 'REF_SEQ_ID', 'REF_POS', 'REF_LEN' ]
    prim_cols = db.open_tab( "PRIMARY_ALIGNMENT" ).make_cursor().open( column_list )
    for row in row_gen( prim_cols, row_range ) :
        cigar = row[ 'CIGAR_LONG' ]
        read = row[ 'READ' ]
        refname = row[ 'REF_SEQ_ID' ]
        refpos = row[ 'REF_POS' ][ 0 ]
        reflen = row[ 'REF_LEN' ][ 0 ]
        for c in cigar_events( cigar, read, refname, refpos, reflen ) :
            yield c
        print

#------------------------------------------------------------------------------------------------------------
def refvar_consumer( q, filename ) :
    print( "refvar_consumer() started" )
    d = {}
    while True :
        signature = q.get()
        if signature == None :
            break
        d[ signature ] = d.get( signature, 0 ) + 1

    f = open( filename, "w" )
    for k, v in sorted( [ ( value, key ) for ( key, value ) in d.items() ], reverse=True ) :
        f.write( "%d %s\n" %( k, v ) )
    f.close()
    print( "refvar_consumer() done" )


#------------------------------------------------------------------------------------------------------------
def allel_consumer( mgr, acc, q_in, q_out1, q_out2 ) :
    print( "allel_consumer() started" )
    try :
        ref_list = mgr.OpenDB( acc ).ReferenceList()
        curr_ref = None
        ref_bases = None
        while True :
            t = q_in.get()
            if t == None :
                break
            ( ref_name, ref_pos, del_len, bases ) = t
            if curr_ref == None or curr_ref != ref_name :
                curr_ref = ref_name
                try :
                    ref_obj = ref_list.find( ref_name )
                    ref_bases = ref_obj.Read( 0, ref_obj.SeqLength() )[:]
                except vdb_error as e :
                    print( e )

            sig1 = "%s:%d:%d:%s" % ( ref_name, ref_pos, del_len, bases )
            q_out1.put( sig1 )
            
            if len( bases ) > 0 :
                # we have insertion/mismatch : let's canonicalize it
                ref_var = mgr.RefVariation( ref_bases, ref_pos, del_len, bases )
                ( a_bases, a_len, a_pos ) = ref_var.GetAllele()
                sig2 = "%s:%d:%d:%s" % ( ref_name, a_pos, a_len, a_bases )
            else :
                # we have a pure deletion
                sig2 = sig1

            q_out2.put( sig2 )
    except vdb_error as e :
        print( e )
    q_out1.put( None )
    q_out2.put( None )
    print( "allel_consumer() done" )


#------------------------------------------------------------------------------------------------------------    
def row_consumer_allel_producer( q_in, q_out ) :
    print( "row_consumer() started" )
    while True :
        row = q_in.get()
        if row == None :
            break
        try :
            cigar = row[ 'CIGAR_LONG' ]
            read = row[ 'READ' ]
            refname = row[ 'REF_SEQ_ID' ]
            refpos = row[ 'REF_POS' ][ 0 ]
            for c in cigar_events( cigar, read, refname, refpos ) :
                q_out.put( c )
        except vdb_error as e :
            print( e )

    print( "row_consumer() done" )
    q_out.put( None )


#------------------------------------------------------------------------------------------------------------
def row_producer( mgr, acc, row_range, q ) :
    try :
        print( "row_producer() started" )
        db = mgr.OpenDB( acc )
        cols = [ 'CIGAR_LONG', 'READ', 'REF_SEQ_ID', 'REF_POS' ]
        prim_cols = db.OpenTable( "PRIMARY_ALIGNMENT" ).CreateCursor().OpenColumns( cols )
        for row in row_gen( prim_cols, row_range ) :
            q.put( row )
        q.put( None )
        print( "row_producer() done" )
    except vdb_error as e :
        print( e )


def process_accession( mgr, acc, row_range = None ) :
    row_q = multiprocessing.Queue()

    p_row_producer = multiprocessing.Process( target = row_producer, args = ( mgr, acc, row_range, row_q ), )

    allel_q = multiprocessing.Queue()
    
    p_row_cons = multiprocessing.Process( target = row_consumer_allel_producer, args = ( row_q, allel_q ), )    
    
    refvar_q1 = multiprocessing.Queue()
    refvar_q2 = multiprocessing.Queue()
    
    p_allel_cons = multiprocessing.Process( target = allel_consumer, args = ( mgr, acc, allel_q, refvar_q1, refvar_q2 ), )

    p_refvar1 = multiprocessing.Process( target = refvar_consumer, args = ( refvar_q1, "ref_var_1.txt" ), )
    p_refvar2 = multiprocessing.Process( target = refvar_consumer, args = ( refvar_q2, "ref_var_2.txt" ), )
    
    p_row_producer.start()
    p_row_cons.start()
    p_allel_cons.start()
    p_refvar1.start()
    p_refvar2.start()
    
    p_row_producer.join()    
    p_row_cons.join()
    p_allel_cons.join()
    p_refvar1.join()
    p_refvar2.join()


#------------------------------------------------------------------------------------------------------------
if __name__ == '__main__':
    lib_r = "./2.8.0/libncbi-vdb.so.2.8.0"
    ACC = "SRR1531793"
    
    if len( sys.argv ) > 1 :
        ACC = sys.argv[ 1 ]

    mgr = manager( OpenMode.Read, lib_r )
    process_accession( mgr, ACC )
    #f1( mgr )



#!/usr/bin/env python

import sys, argparse, vdb

PY3 = sys.version_info[ 0 ] == 3

if PY3 :
    def xrange( *args, **kwargs ) :
        return iter( range( *args, **kwargs ) )

def fastq_from_tbl( args, tbl ) :
    acc = args.accession[ 0 ]
    
    col_names = [ "READ", "(INSDC:quality:text:phred_33)QUALITY", "NAME" ]
    if args.split :
        col_names.append( "READ_START" )
        col_names.append( "READ_LEN" )
    cols = tbl.CreateCursor().OpenColumns( col_names )
    
    c_read = cols[ col_names[ 0 ] ]
    c_qual = cols[ col_names[ 1 ] ]
    c_name = cols[ col_names[ 2 ] ]
    if args.split :
        c_read_start = cols[ col_names[ 3 ] ]
        c_read_len   = cols[ col_names[ 4 ] ]
    
    first, count = c_read.row_range()
    
    if args.first != None :
        first = args.first[ 0 ]
    if args.count != None :
        count = args.count[ 0 ]

    if args.split :
        fastq = '@{0}.{1}.{2} length={3}\n{4}\n+{0}.{1}.{2} length={3}\n{5}'
        for row in xrange( first, first + count ) :
            name     = c_name.Read( row )
            read     = c_read.Read( row )
            qual     = c_qual.Read( row )
            rd_start = c_read_start.Read( row )
            rd_len   = c_read_len.Read( row )
            for x in xrange( 0, len( rd_start ) ) :
                rlen  = rd_len[ x ]
                if rlen > 0 :
                    start = rd_start[ x ]
                    end   = start + rlen
                    print( fastq.format( acc, name, x+1, rlen, read[ start:end ], qual[ start:end ] ) )
    else :
        fastq = '@{0}.{1} length={2}\n{3}\n+{0}.{1} length={2}\n{4}'
        for row in xrange( first, first + count ) :
            read = c_read.Read( row )
            print( fastq.format( acc, c_name.Read( row ), len( read ), read, c_qual.Read( row ) ) )

    
if __name__ == '__main__' :
    parser = argparse.ArgumentParser()
    parser.add_argument( 'accession', nargs='*' )    
    parser.add_argument( '-X', '--first', metavar='row-id', help='first row-id', nargs=1, type=int, dest='first' )
    parser.add_argument( '-N', '--count', metavar='rows', help='how many reads', nargs=1, type=int, dest='count' )
    parser.add_argument( '--split', help='split spot', action='store_true' )
    args = parser.parse_args()
    
    try :
        #open a manager in read-mode ( dflt-mode )
        mgr = vdb.manager()
        
        for acc in args.accession :
            #detect path-type ( database or table or anything-else )
            pt = mgr.PathType( acc )
            if pt == vdb.PathType.Database :
                #object is a database
                fastq_from_tbl( args, mgr.OpenDB( acc ).OpenTable( "SEQUENCE" ) )
            elif pt == vdb.PathType.Table :
                #object is a table
                fastq_from_tbl( args, mgr.OpenTable( acc ) )
            else :
                print( "%s is not an SRA-object"%( acc ) )
    except vdb.vdb_error as e :
        print( e )
    except KeyboardInterrupt :
        print( "^C" )



#!/usr/bin/env python

from vdb import *

lib_wr = "./2.8.0/libncbi-wvdb.so.2.8.0"

schematxt = '''
version 1;
table A_TABLE #1.0
{
    column U8 C1;
    column U32 C2;
    column ascii C3;
    column I16 C4;
    column F32 C5;
    column bool C6;
};

''' 

def fill_table_random( cur, cols, row_count, value_count ) :
    for idx in xrange( 0, row_count ) :
        cur.OpenRow()
        cols[ "C1" ].write_rand( value_count, 100 )
        cols[ "C2" ].write_rand( value_count, 100000 )
        cols[ "C3" ].write_rand( value_count )
        cols[ "C4" ].write_rand( value_count, 100000 )
        cols[ "C5" ].write_rand( value_count, 100000 )
        cols[ "C6" ].write_rand( value_count )
        cur.CommitRow()
        cur.CloseRow()

def fill_table_with_values( cur, cols ) :
    cur.OpenRow()
    cols[ "C1" ].write( [ 1, 2, 3, 4 ] )
    cols[ "C2" ].write( [ 1000, 1001, 1002 ] )
    cols[ "C3" ].write( "hallo" )
    cols[ "C4" ].write( [ 101010, 2020, 3030 ] )
    cols[ "C5" ].write( [ 10.101, 20.202, 30.303 ] )
    cols[ "C6" ].write( [ True, True, False, False, True ] )
    cur.CommitRow()
    cur.CloseRow()

def fill_table_with_single_values( cur, cols ) :
    cur.OpenRow()
    cols[ "C1" ].write( 1 )
    cols[ "C2" ].write( 1000 )
    cols[ "C3" ].write( "hallo" )
    cols[ "C4" ].write( 5544 )
    cols[ "C5" ].write( 55.5 )
    cols[ "C6" ].write( True )
    cur.CommitRow()
    cur.CloseRow()

def fill_table_with_default_values( cur, cols, row_count ) :
    cols[ "C1" ].set_default( [ 100, 101 ] )
    for idx in xrange( 0, row_count ) :
        cur.OpenRow()
        cols[ "C2" ].write( [ 10 + idx, 11 + idx, 12 + idx ] )
        cols[ "C3" ].write( "line #%d"%idx )
        cols[ "C4" ].write( [ 101 + idx, 102 + idx, 103 + idx ] )
        cols[ "C5" ].write( [ 10.1 + idx, 20.2 + idx, 30.3 + idx ] )
        cols[ "C6" ].write( [ True ] )
        cur.CommitRow()
        cur.CloseRow()

def make_table( mgr, schema_txt, table_name ) :
    try :
        schema = mgr.MakeSchema( schema_txt )
        tbl = mgr.CreateTable( schema, "A_TABLE", table_name )
        cur = tbl.CreateCursor( OpenMode.Write )
        cols = cur.OpenColumns( [ "C1", "C2", "C3", "C4", "C5", "C6" ] )
        fill_table_random( cur, cols, 2, 5 )
        fill_table_with_values( cur, cols )
        fill_table_with_single_values( cur, cols )
        fill_table_with_default_values( cur, cols, 2 )
        cur.Commit()
    except vdb_error as e :
        print( e )

        
if __name__ == '__main__' :
    
    table_name = "L3"
        
    try :
        #open a manager
        mgr = manager( OpenMode.Write, lib_wr )
    
        make_table( mgr, schematxt, table_name )
        mgr.OpenTable( table_name ).print_rows()

    except vdb_error as e :
        print( e )



#!/usr/bin/env python

from vdb import *

lib_wr = "./2.8.0/libncbi-wvdb.so.2.8.0"

schematxt = '''
version 1;
table A_TABLE #1.0
{
    column U32 C1;
    column ascii C2;
};

''' 

def fill_table_random( cur, cols, kidx, row_count, value_count ) :
    for idx in xrange( 0, row_count ) :
        cur.OpenRow()
        row_id = cur.RowId()
        cols[ "C1" ].write_rand( value_count, 0xffffffff )
        rand_str = random_string( value_count )
        cols[ "C2" ].write( rand_str )
        cur.CommitRow()
        cur.CloseRow()
        #print( "Row = %d"%row_id )
        if ( idx % 1000 ) == 0 :
            kidx.InsertText( True, rand_str, row_id )

def make_table( mgr, schema_txt, table_spec, table_name ) :
    try :
        schema = mgr.MakeSchema( schema_txt )
        tbl = mgr.CreateTable( schema, table_spec, table_name )
        kidx = tbl.CreateIndex( "C2_IDX", IndexType.Text )
        cur = tbl.CreateCursor( OpenMode.Write )
        cols = cur.OpenColumns( [ "C1", "C2" ] )
        fill_table_random( cur, cols, kidx, 200000, 25 )
        kidx.Commit()
        cur.Commit()
    except vdb_error as e :
        print( e )


if __name__ == '__main__' :
    
    table_name = 'L9'
    table_spec = 'A_TABLE'
    
    try :
        #open a manager
        mgr = manager( OpenMode.Write, lib_wr )
    
        make_table( mgr, schematxt, table_spec, table_name )
        #mgr.OpenTable( table_name ).print_rows()

    except vdb_error as e :
        print( e )



from ctypes import (c_int, c_char_p, cdll, Structure, c_uint32, ARRAY,
                    c_char, POINTER, c_size_t, c_void_p, c_uint64, byref,
                    create_string_buffer, c_uint)
import platform

import os


def check_rc(rc):
    if rc:
        raise RuntimeError("bad rc, %d", rc)
    return 1

def get_lib_name():
    if platform.system() == "Windows":
        return "ncbi-vdb.dll"
    elif platform.system() == "Darwin":
        return "libncbi-vdb.dylib"
    else:
        return "libncbi-vdb.so"


_krypto = _kfs = cdll.LoadLibrary( get_lib_name() )


class _KKey(Structure):
    _fields_ = [("type", c_uint32),
                ("text", ARRAY(c_char, 32))]


def errcheck(result, func, args):
    return check_rc(result)

# setting up declarations for functions
KKeyInitRead = _krypto.KKeyInitRead
KKeyInitRead.restype = c_uint
KKeyInitRead.argtypes = [POINTER(_KKey), c_int, c_char_p, c_size_t]
KKeyInitRead.errcheck = errcheck

KEncFileMakeRead = _krypto.KEncFileMakeRead
KEncFileMakeRead.restype = c_uint
KEncFileMakeRead.argtypes = (POINTER(c_void_p), c_void_p, POINTER(_KKey))
KEncFileMakeRead.errcheck = errcheck

KEncryptFileMakeRead = _krypto.KEncryptFileMakeRead
KEncryptFileMakeRead.restype = c_uint
KEncryptFileMakeRead.argtypes = (POINTER(c_void_p), c_void_p, POINTER(_KKey))
KEncryptFileMakeRead.errcheck = errcheck

KEncFileMakeUpdate = _krypto.KEncFileMakeUpdate
KEncFileMakeUpdate.restype = c_uint
KEncFileMakeUpdate.argtypes = (POINTER(c_void_p), c_void_p, POINTER(_KKey))
KEncFileMakeUpdate.errcheck = errcheck

KEncFileMakeWrite = _krypto.KEncFileMakeWrite
KEncFileMakeWrite.restype = c_uint
KEncFileMakeWrite.argtypes = (POINTER(c_void_p), c_void_p, POINTER(_KKey))
KEncFileMakeWrite.errcheck = errcheck

KReencFileMakeRead = _krypto.KReencFileMakeRead
KReencFileMakeRead.restype = c_uint
KReencFileMakeRead.argtypes = [POINTER(c_void_p), c_void_p, POINTER(_KKey), POINTER(_KKey)]
KReencFileMakeRead.errcheck = errcheck

KFileMakeFDFileRead = _kfs.KFileMakeFDFileRead
KFileMakeFDFileRead.restype = c_uint
KFileMakeFDFileRead.argtypes = [POINTER(c_void_p), c_int]
KFileMakeFDFileRead.errcheck = errcheck

KFileMakeFDFileWrite = _kfs.KFileMakeFDFileWrite
KFileMakeFDFileWrite.restype = c_uint
KFileMakeFDFileWrite.argtypes = [POINTER(c_void_p), c_int, c_int]
KFileMakeFDFileWrite.errcheck = errcheck

KFileRelease = _kfs.KFileRelease
KFileRelease.argtypes = [c_void_p]

KFileReadAll = _kfs.KFileReadAll
KFileReadAll.restype = c_uint
KFileReadAll.argtypes = [c_void_p, c_uint64, c_void_p, c_size_t, POINTER(c_size_t)]
KFileReadAll.errcheck = errcheck

KFileWrite = _kfs.KFileWrite
KFileWrite.restype = c_uint
KFileWrite.argtypes = [c_void_p, c_uint64, c_char_p, c_size_t, POINTER(c_size_t)]
KFileWrite.errcheck = errcheck

KFileWriteAll = _kfs.KFileWriteAll
KFileWriteAll.restype = c_uint
KFileWriteAll.argtypes = [c_void_p, c_uint64, c_char_p, c_size_t, POINTER(c_size_t)]
KFileWriteAll.errcheck = errcheck

KFileSize = _kfs.KFileSize
KFileSize.restype = c_uint
KFileSize.argtypes = [c_void_p, POINTER(c_uint64)]
KFileSize.errcheck = errcheck

KFileSetSize = _kfs.KFileSetSize
KFileSetSize.restype = c_uint
KFileSetSize.argtypes = [c_void_p, c_uint64]
KFileSetSize.errcheck = errcheck

_kkeyAES128 = 1

class _KFile(object):
    def __init__(self):
        self._kfile = None

    def __dealloc__(self):
        if self._kfile:
            KFileRelease(self._kfile)

    def __enter__(self):
        return self

    def __exit__(self, t, value, traceback):
        self.close()

    def close(self):
        if self._kfile:
            KFileRelease(self._kfile)
            self._kfile = None

    def write_all(self, buf, offset):
        """
        Write `buf` bytes into file at given `offset`
        :param buf: Byte-string to write.
        :param offset: Offset position in bytes to start writing at
        """
        if not self._kfile:
            raise ValueError("I/O operation on closed file")
        written = c_size_t()
        KFileWriteAll(self._kfile, offset, buf, len(buf), byref(written))
        return written.value

    def read_all(self, size, offset):
        """
        Read `size` bytes from `offset`, might return less than `size` bytes if EOF is reached before.

        :param size: Number of bytes to read
        :param offset: Offset to start read at
        :return: A byte-buffer containing read bytes
        """
        if not self._kfile:
            raise ValueError("I/O operation on closed file")
        read = c_size_t()
        buf = create_string_buffer(size)
        KFileReadAll(self._kfile, offset, buf, size, byref(read))
        # return buf.value[0:read.value]
        return buf[0:read.value]

    @property
    def size(self):
        """ Size of the file in bytes
        """
        if not self._kfile:
            raise ValueError("I/O operation on closed file")
        sz = c_uint64()
        KFileSize(self._kfile, byref(sz))
        return sz.value


class _KFileStream(_KFile):
    def __init__(self):
        super(_KFileStream, self).__init__()
        self.pos = 0

    def write(self, buf):
        """ Write `buf` into current position of the file.

        :param buf: Byte-string to write into file
        """
        left = len(buf)
        total_written = 0
        while left:
            written = self.write_all(buf[total_written:], self.pos)
            self.pos += written
            total_written += written
            left -= written
        return total_written

    def seek(self, pos):
        """ Sets current position in the file """
        self.pos = pos

    def read(self, size=-1):
        """ Read `size` bytes from file
        """
        strings = []
        if not self._kfile:
            raise ValueError("I/O operation on closed file")
        if size == -1:
            strings = []
            while 1:
                s = self.read_all(16 * 1024, self.pos)
                self.pos += len(s)
                if not s:
                    break
                strings.append(s)
            return b''.join(strings)
        else:
            res = self.read_all(size, self.pos)
            self.pos += len(res)
            return res

    def truncate(self, size=None):
        """
        Truncate file up to current position (default) or up to `size`
        """
        if size is None:
            size = self.pos
        KFileSetSize(self._kfile, size)


class Decryptor(_KFileStream):
    """
    Create a read-only file-like object that will decrypt `src_file` using `password`.

    :param password: Byte string with password for decryption.
    :param src_file: Integer file descriptor of input file.
    :param close: If true `src_file` will be closed when this object is closed.
    """

    def __dealloc__(self):
        if self.src_file:
            os.close(self.src_file)

    def __init__(self, password, src_file, close=True):
        super(Decryptor, self).__init__()
        self.kencfile = c_void_p()
        self.src_file = src_file
        self._close = close
        key = _KKey()
        kfile = c_void_p()
        KKeyInitRead(byref(key), _kkeyAES128, password, len(password))
        KFileMakeFDFileRead(byref(kfile), src_file)
        kencfile = c_void_p()
        try:
            KEncFileMakeRead(byref(kencfile), kfile, byref(key))
        except:
            KFileRelease(kfile)
            raise
        self._kfile = kencfile

    def close(self):
        """
        Releases memory and closes `src_file` if was requested so in constructor.
        """
        super(Decryptor, self).close()
        if self.src_file and self._close:
            os.close(self.src_file)
            self.src_file = None


class Encryptor(_KFileStream):
    """
    Create a file-like object that will encrypt input data and write it into `out_file` file descriptor.

    Call `write` method on this object to write encrypted data into out_file.

    :param password: Byte string with password.
    :param out_file: Integer file descriptor of output file.
    :param close: If true `out_file` descriptor will be closed when this object is closed.
    :param update: If true encrypted file can be appended/modified, otherwise it will be truncated.
    """

    def __dealloc__(self):
        if self.src_file and self._close:
            os.close(self.src_file)

    def __init__(self, password, out_file, close=True, update=False):
        super(Encryptor, self).__init__()
        kfile = c_void_p()
        self.key = _KKey()
        self.out_file = out_file
        kencfile = c_void_p()
        self._close = close
        KKeyInitRead(byref(self.key), _kkeyAES128, password, len(password))
        KFileMakeFDFileWrite(byref(kfile), 1 if update else 0, out_file)
        try:
            if update:
                KEncFileMakeUpdate(byref(kencfile), kfile, byref(self.key))
            else:
                KEncFileMakeWrite(byref(kencfile), kfile, byref(self.key))
        except:
            KFileRelease(kfile)
            raise
        self._kfile = kencfile

    def close(self):
        """ Releases memory and closes `out_file` if was requested so in constructor.
        """
        super(Encryptor, self).close()
        if self.out_file and self._close:
            os.close(self.out_file)
            self.out_file = None

    def flush(self):
        # TODO: implement this
        pass


class Reencryptor(_KFileStream):
    """
    Create a read-only file-like object that will decrypt `src_file` using `dec_password` and then
    encrypt it with `enc_password`.  This objects acts like a decryptor attached to encryptor.

    :param dec_password: Byte-string of decryption password used to decrypt `src_file`.
    :param enc_password: Byte-string of encryption password used to encrypt returned data.
    :param src_file: An integer file descriptor of input file, it should have read operation allowed.
    :param close: If true `src_file` descriptor will be closed when this object is closed.
    """

    def __init__(self, dec_password, enc_password, src_file, close=True):
        super(Reencryptor, self).__init__()
        kfile = c_void_p()
        dec_key = _KKey()
        enc_key = _KKey()
        self.src_file = src_file
        kencfile = c_void_p()
        self._close = close
        KKeyInitRead(byref(dec_key), _kkeyAES128, dec_password, len(dec_password))
        KKeyInitRead(byref(enc_key), _kkeyAES128, enc_password, len(enc_password))
        KFileMakeFDFileRead(byref(kfile), src_file)
        try:
            KReencFileMakeRead(byref(kencfile), kfile, byref(dec_key), byref(enc_key))
        except:
            KFileRelease(kfile)
            raise
        self._kfile = kencfile

    def __dealloc__(self):
        if self.src_file and self._close:
            os.close(self.src_file)

    def close(self):
        """
        Releases memory and closes `src_file` if was requested so in constructor
        """
        super(Reencryptor, self).close()
        if self.src_file and self._close:
            os.close(self.src_file)
            self.src_file = None


class FDFileReader(_KFileStream):
    """
    Deprecated, do not use.  Will be removed in 1.6
    Create a read-only file-like object that will read `f` file-object

    :param f: Source file-like object, should have `fileno` method.
    :param close: If true will close `f` when this object is closed
    """

    def __init__(self, f, close=True):
        super(FDFileReader, self).__init__()
        self._f = f
        kfile = c_void_p()
        KFileMakeFDFileRead(byref(kfile), f.fileno())
        self._kfile = kfile
        self._close = close

    def close(self):
        super(FDFileReader, self).close()
        if self._close:
            self._f.close()


class EncryptingReader(_KFileStream):
    """
    Create a read-only file-like object that will read and encrypt `src_file` using `password`

    :param src_file: An integer file descriptor to be encrypted.
    :param password: Byte-string password used to encrypt.
    :param close: If true `src_file` descriptor will be closed when this object is closed
    """

    def __init__(self, src_file, password, close=True):
        enc_key = _KKey()
        KKeyInitRead(byref(enc_key), _kkeyAES128, password, len(password))
        kencfile = c_void_p()
        KEncryptFileMakeRead(byref(kencfile), src_file._kfile, byref(enc_key))
        self._kfile = kencfile
        self._close = close
        self._src_file = src_file

    def close(self):
        """
        Releases memory and closes `src_file` if it was requested so in constructor
        """
        super(EncryptingReader, self).close()
        if self._close:
            self._src_file.close()



#!/usr/bin/env python

import vdb, csv, os

schematxt = '''
version 1;
table CSVTAB #1.0
{
    column U16 C1;
    column ascii C2;
};
'''

def write_csv( filename, rowcount ) :
    with open( filename, 'w' ) as f :
        for i in range( 1, rowcount ) :
            f.write( "%d, line # %d\n"%( i, i ) )
        f.close()
        
def csv_to_vdb( mgr, csv_file, spec, vdb_table ) :
    with open( csv_file, 'r' ) as f :
        spec = 'CSVTAB'
        tbl_wr = mgr.CreateTable( mgr.MakeSchema( schematxt ), spec, vdb_table )
        cur = tbl_wr.CreateCursor( vdb.OpenMode.Write )
        cols = cur.OpenColumns( [ "C1", "C2" ] )
        for row in csv.reader( f ) :
            cur.OpenRow()
            cols[ "C1" ].write( [ int( row[ 0 ] ) ] )
            cols[ "C2" ].write( row[ 1 ].strip() );
            cur.CommitRow()
            cur.CloseRow()
        cur.Commit()

if __name__ == '__main__':
    csv_file = 'data.txt'
    write_csv( csv_file, 10 )
    try :
        mgr = vdb.manager( vdb.OpenMode.Write, "./2.8.0/libncbi-wvdb.so.2.8.0" )
        spec = 'CSVTAB'
        csv_to_vdb( mgr, csv_file, spec, 'L8' )
        mgr.OpenTable( spec ).print_rows()
    except vdb.vdb_error as e :
        print( e )
    os.remove( csv_file )



import sys, os
from collections import OrderedDict
from operator import itemgetter

def inspect( filename ) :
    if os.path.isfile( filename ) :
        print( "inspecting: %s"%( filename ) )

        max = 0
        min = 0
        sum = 0
        cnt = 0
        
        d = {}
        
        with open( filename ) as f :
            for line in f :
                v = line.strip().split()
                if v[ 0 ] == "R" and len( v ) > 3:

                    req_pos = int( v[ 1 ] )
                    if req_pos in d.keys() :
                        d[ req_pos ] = d[ req_pos ] + 1
                    else :
                        d[ req_pos ] = 1
                        
                    req_len = int( v[ 3 ] )                        
                    sum += req_len
                    cnt += 1
                    if req_len > max :
                        max = req_len;
                    if min == 0 :
                        min = req_len
                    elif req_len < min :
                        min = req_len

        print( "\treq\t{:,d}".format( cnt ) )
        print( "\tmin\t{:,d}".format( min ) )
        print( "\tmax\t{:,d}".format( max ) )
        print( "\tsum\t{:,d}".format( sum ) )

        d1 = {}
        repeats = 0
        for k_pos, v_cnt in d.iteritems() :
            if v_cnt > 1 :
                d1[ k_pos ] = v_cnt
                repeats += v_cnt
        print( "\trepeats\t{:,d}".format( repeats ) )
        
        od = OrderedDict( reversed( sorted( d1.items(), key = itemgetter( 1 ) ) ) )
        i = 0
        for k_pos, v_cnt in od.iteritems() :
            print( "\tpos {:,d}\t{:d} times".format( k_pos, v_cnt ) )
            i += 1
            if i > 5 :
                break
        print( " " )

if __name__ == '__main__':
    if len( sys.argv ) > 1 :
        inspect( "%s.rec"%( sys.argv[ 1 ] ) )
        inspect( "%s.inner.rec"%( sys.argv[ 1 ] ) )        
        inspect( "%s.outer.rec"%( sys.argv[ 1 ] ) )        
    else :
        print( "no input given" )
        print( "example: python inspect_rec.py SRR341578" )



#!/usr/bin/env python

from vdb import *

def get_one_value_v1( mgr, acc, column_name, row_id ) :
    try :
        tbl = mgr.OpenTable( acc )
        cur = tbl.CreateCursor()
        col = cur.OpenColumns( column_name )
        return col.Read( row_id )
    except vdb_error as e :
        print( e )

def get_one_value_v2( mgr, acc, column_name, row_id ) :
    try :
        return mgr.OpenTable( acc ).CreateCursor().OpenColumns( column_name ).Read( row_id )
    except vdb_error as e :
        print( e )

def get_readable_columns( mgr, acc ) :
    try :
        tbl = mgr.OpenTable( acc )
        return tbl.ListCol()
    except vdb_error as e :
        print( e )

def print_column_infos( mgr, acc, column_names ) :
    try :
        tbl = mgr.OpenTable( acc )
        cur = tbl.CreateCursor()
        cols = cur.OpenColumns( column_names )
        try :
            v = cols.itervalues()
        except AttributeError :
            v = cols.values()
        for c in v :
            print( "%s\t %s"%( c.name, c.row_range() ) )
            print( "domain: %s, bits: %d, dim : %d"%( c.domain(), c.bits(), c.dim() ) )
    except vdb_error as e :
        print( e )

def inspect_column_values( mgr, acc, column_names, row_id ) :
    try :
        tbl = mgr.OpenTable( acc )
        cur = tbl.CreateCursor()
        cols = cur.OpenColumns( column_names )
        
        #retrieve a cell-value as an array of Uint32-values ( in this case a single value )
        spot_id = cols[ "SPOT_ID" ].Read( row_id )
        print( "SPOT_ID[1] = %s"%( spot_id ) )
        
        #retrieve a cell-value as a string
        read = cols[ "READ" ].Read( row_id )
        print( "READ[1] = %s"%( read ) )
        
        #retrieve a cell-value as an array of Uint8-values
        q = cols[ "QUALITY" ].Read( row_id )
        print( "QUALITY[1] = %s"%( q ) )
        print( "avg of quality values = %f"%( sum( q ) / float( len( q ) ) ) )

    except vdb_error as e :
        print( e )

if __name__ == '__main__' :
    
    #open a manager in read-mode
    mgr = manager()
    
    #a long and a short way of reading a value from a table
    print( get_one_value_v1( mgr, "SRR000002", "READ", 2000 ) )
    print( get_one_value_v2( mgr, "SRR000002", "READ", 2000 ) )
    
    #print readable columns
    print( get_readable_columns( mgr, "SRR000001" ) )

    #print details about some columns
    print_column_infos( mgr, "SRR000001", [ "SPOT_ID", "READ", "QUALITY" ] )

    #open multiple columns at the same time
    inspect_column_values( mgr, "SRR000001", [ "SPOT_ID", "READ", "QUALITY" ], 1 )



#!/usr/bin/env python
import cipher
import getpass
from optparse import OptionParser
import shutil
import sys


if __name__ == '__main__':
    version = '%prog 1.0'
    parser = OptionParser(usage='''usage: %prog [options] INFILE OUTFILE
    Encrypts file''', version=version)
    parser.add_option('-P', '--password', dest='password', default=None)
    args, values = parser.parse_args()
    if len(values) < 2:
        parser.print_usage()
        sys.exit(2)
    in_name, out_file = values
    if not args.password:
        args.password = getpass.getpass()

    with open(in_name, 'rb')  as in_f:
        with open(out_file, 'wb') as out_f:
            with cipher.Encryptor(args.password.encode('ascii'), out_f.fileno(), close=False, update=False) as enc:
                shutil.copyfileobj(in_f, enc)



#!/usr/bin/env python

from vdb import *

def print_repolist( repos ) :
    for r in repos :
        name = r.name()
        print "%s.displayname = %s"%( name, r.display_name() )
        print "%s.category = %s"%( name, RepoCat2String( r.category() ) )
        print "%s.subcategory = %s"%( name, RepoSubCat2String( r.sub_category() ) )
        print "%s.root = %s"%( name, r.get_root() )
        print "%s.resolver = %s"%( name, r.resolver() )
        print "%s.disabled = %s"%( name, r.is_disabled() )
        print "%s.cache enabled = %s"%( name, r.cache_enabled() )
        print "-"

def report_repos( rm ) :
    print "user repos:"
    print_repolist( rm.user_repos() )
    print "site repos:"
    print_repolist( rm.site_repos() )
    print "remote repos:"
    print_repolist( rm.remote_repos() )

if __name__ == '__main__':
    lib_263_rd = "./2.6.3/libncbi-vdb.so.2.6.3"
    lib_262_rd = "./2.6.2/libncbi-vdb.so.2.6.2"
    lib_rd = "/home/raetzw/.ncbi/lib64/libncbi-vdb.so"
    
    mgr = manager( OpenMode.Read, lib_262_rd )
    cfg = mgr.make_config()

    rm = cfg.make_repo_mgr()
    print "has remote access = %s"%( rm.has_remote_access() )

    #report_repos( rm )
    
    print "user.disabled = %s"%( rm.cat_disabled( RepoCat.User ) )
    print "site.disabled = %s"%( rm.cat_disabled( RepoCat.Site ) )
    print "remote.disabled = %s"%( rm.cat_disabled( RepoCat.Remote ) )
    
    vfs_mgr = mgr.make_vfs_mgr()
    resolv = vfs_mgr.make_resolver( cfg )
    #resolv. remote_enable( ResolvEnable.AlwaysEnable )

    print "local  = %s"%( resolv.query_local( vfs_mgr.make_path( "ncbi-acc:SRR001000" ) ) )
    print "remote = %s"%( resolv.query_remote( vfs_mgr.make_path( "ncbi-acc:SRR001000" ) ) )
    print "cache  = %s"%( resolv.query_cache( vfs_mgr.make_path( "ncbi-acc:SRR001000" ) ) )    
